package com.gct.dataengine.central.config;

import com.gct.dataengine.central.job.SaveMonitorViewInfo;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * class name: QuartzTaskConfig
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/5/21 上午10:21
 */
@Configuration
public class QuartzTaskConfig {

    private static final   String jobName = "SaveMonitorViewInfoJob";

    private static final   String jobGroup = "SAVEMONITORVIEWINFOJOB_GROUP";


    @Bean
    public JobDetail SaveMonitorViewInfoJob() {
        try {
            return JobBuilder.newJob(SaveMonitorViewInfo.class)
                    .withIdentity(jobName, jobGroup)
                    .storeDurably()
                    .build();
        }catch (Exception e)
        {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", jobName, jobGroup,e.getMessage());
            throw e;
        }

    }

    @Bean
    public Trigger SaveMonitorViewInfoJobTrigger(){
        try {
            SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule()
                    .withIntervalInMinutes(2);
            scheduleBuilder.repeatForever();
            return TriggerBuilder.newTrigger()
                    .forJob(jobName, jobGroup)
                    .startNow()
                    .withSchedule(scheduleBuilder).build();
        }catch (Exception e)
        {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}",jobName, jobGroup, e.getMessage());
            throw e;
        }
    }
}
