package com.gct.dataengine.central.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.common.util.StringUtil;
import com.gct.dataengine.central.entity.MachineServerState;
import com.gct.dataengine.central.entity.MonitorViewInfoList;
import com.gct.dataengine.central.entity.ViewTemplateInfo;
import com.gct.dataengine.central.entity.vo.GetMonitorInfoVo;
import com.gct.dataengine.central.feignclient.SuitcaseClient;
import com.gct.dataengine.central.mapper.MonitorViewInfoListMapper;
import com.gct.dataengine.central.service.MachineServerStateService;
import com.gct.dataengine.central.service.MonitorViewInfoListService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.dataengine.central.service.ViewTemplateInfoService;
import com.google.gson.Gson;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 监控详情信息表 服务实现类
 * </p>
 *
 * @author gxt
 * @since 2022-07-27
 */
@Service
public class MonitorViewInfoListServiceImpl extends ServiceImpl<MonitorViewInfoListMapper, MonitorViewInfoList> implements MonitorViewInfoListService {

    @Autowired
    MachineServerStateService machineServerStateService;

    @Autowired
    ViewTemplateInfoService viewTemplateInfoService;

    @Autowired
    SuitcaseClient suitcaseClient;

    @Override
    public boolean saveMonitorViewInfo() {
        List<ViewTemplateInfo> viewList = viewTemplateInfoService.getList();
        List<MachineServerState> machineList = machineServerStateService.getList();
        List<MonitorViewInfoList> list = new ArrayList<>();
        if (viewList.isEmpty()) return false;
        viewList.forEach(item -> {
            machineList.forEach(x -> {
                if (item.getMachineTemplateId().equals(x.getTemplateId())) {
                    list.add(new MonitorViewInfoList()
                            .setId(item.getId())
                            .setMachineTemplateId(item.getMachineTemplateId())
                            .setMonitorAlias(item.getViewTempName())
                            .setImgPath(item.getHeaderImg())
                            .setAccessStatus(x.getOnline())
                            .setGroupId(item.getGroupId())
                            .setIp(x.getIpAddress())
                            .setPort(Integer.parseInt(x.getPort()))
                            .setMachineUsable(item.getEnableState())
                            .setCommunicateDetail(x.getProtocol())
                            .setAddress(item.getAddress())
                            .setLastAccessTime(LocalDateTime.now())
                            .setRemark(item.getTagInfo()));
                }
            });
        });
        return saveOrUpdateBatch(list);
    }

    @Override
    public Page<MonitorViewInfoList> getMonitorViewInfoList(GetMonitorInfoVo getMonitorInfoVo) {
        Gson gson = new Gson();
        LambdaQueryWrapper<MonitorViewInfoList> wrapper = new LambdaQueryWrapper<>();
        if (!StringUtil.isBlankOrEmpty(getMonitorInfoVo.getGroupId())) {
            List list = gson.fromJson(gson.toJson(gson.fromJson(suitcaseClient.getGroupIdList(getMonitorInfoVo.getGroupId(), getMonitorInfoVo.getTempType()), Result.class).getData()), List.class);
            wrapper = wrapper.in(MonitorViewInfoList::getGroupId, list);
        }
        Page<MonitorViewInfoList> page = baseMapper.selectPage(new Page<>(getMonitorInfoVo.getCurPage(), getMonitorInfoVo.getPageSize()), wrapper);
        List<String> ids = new ArrayList<>();
        page.getRecords().forEach(item->{
            ids.add(item.getGroupId());
        });
        Map<String, String> map = gson.fromJson(gson.toJson(gson.fromJson(suitcaseClient.getAllParentName(ids, getMonitorInfoVo.getTempType()), Result.class).getData()), Map.class);
        page.getRecords().forEach(item->{
            if (map.containsKey(item.getGroupId())){
                item.setGroupName(map.get(item.getGroupId()));
            }
        });
        return page;
    }
}
