package com.gct.dataengine.central.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.core.result.Result;
import com.gct.common.util.StringUtil;
import com.gct.dataengine.central.entity.ViewTemplateInfo;
import com.gct.dataengine.central.entity.vo.AddTempInfoVo;
import com.gct.dataengine.central.entity.vo.CopyViewTemplateVo;
import com.gct.dataengine.central.entity.vo.GetTempInfoVo;
import com.gct.dataengine.central.entity.vo.SaveTempInfoVo;
import com.gct.dataengine.central.feignclient.SuitcaseClient;
import com.gct.dataengine.central.mapper.ViewTemplateInfoMapper;
import com.gct.dataengine.central.service.ViewTemplateInfoService;
import com.google.gson.Gson;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 视图模板信息表 服务实现类
 * </p>
 *
 * @author gxt
 * @since 2022-07-27
 */
@Service
public class ViewTemplateInfoServiceImpl extends ServiceImpl<ViewTemplateInfoMapper, ViewTemplateInfo> implements ViewTemplateInfoService {

    @Autowired
    SuitcaseClient suitcaseClient;

    @Override
    public boolean saveTemplateInfo(SaveTempInfoVo saveTempInfoVo) {
        ViewTemplateInfo viewTemplateInfo = new ViewTemplateInfo()
                .setId(saveTempInfoVo.getTemplateId())
                .setViewTempName(saveTempInfoVo.getTempName())
                .setMachineTemplateId(saveTempInfoVo.getEquipmentId())
                .setDeviceId(saveTempInfoVo.getDeviceId())
                .setGroupId(saveTempInfoVo.getGroupId())
                .setHeaderImg(saveTempInfoVo.getImg())
                .setTagInfo(saveTempInfoVo.getTag())
                .setEnableState(2)
                .setAddress(saveTempInfoVo.getAddress())
                .setMachineTemplateIdList(JSON.toJSONString(saveTempInfoVo.getEquipmentBindId()));
        return saveOrUpdate(viewTemplateInfo);
    }

    @Override
    public ViewTemplateInfo getTempplateInfo(GetTempInfoVo getTemplateInfoVo) {
        return getBaseMapper().selectById(getTemplateInfoVo.getId());
    }

    @Override
    public Page<ViewTemplateInfo> getTempplateInfoList(GetTempInfoVo getTempInfoVo) {
        LambdaQueryWrapper<ViewTemplateInfo> wrapper = new LambdaQueryWrapper<>();
        if (!StringUtil.isBlankOrEmpty(getTempInfoVo.getGroupId())){
            wrapper = wrapper.eq(ViewTemplateInfo::getGroupId,getTempInfoVo.getGroupId());
        }
        wrapper = wrapper.orderByAsc(ViewTemplateInfo::getViewTempName);
        Page<ViewTemplateInfo> viewTemplateInfoPage = baseMapper.selectPage(new Page<>(getTempInfoVo.getCurPage(), getTempInfoVo.getPageSize()), wrapper);
        List<Map<String, String>> groupName = getGroupName(getTempInfoVo.getTempType());
        viewTemplateInfoPage.getRecords().forEach(item->{
            groupName.forEach(x->{
                if (!StringUtil.isBlankOrEmpty(x.get(item.getGroupId()))) {
                    item.setGroupName(x.get(item.getGroupId()));
                }
            });
        });
        return viewTemplateInfoPage;
    }

    @Override
    public boolean addTemplateInfo(AddTempInfoVo addTempInfoVo) {
        ViewTemplateInfo viewTemplateInfo = new ViewTemplateInfo();
        BeanUtils.copyProperties(addTempInfoVo,viewTemplateInfo);
        viewTemplateInfo.setEnableState(2);
        return baseMapper.insert(viewTemplateInfo) > 0;
    }

    @Override
    public boolean deleteTemplateInfo(String id) {
        //TODO 这里只是简单的删除 视图信息,oss 中可以通过定时任务来判断 id 没有绑定就删除
        return baseMapper.deleteById(id) > 0;
    }

    @Override
    public boolean updateTemplateInfo(String id,Integer enableState) {
        return baseMapper.updateById(new ViewTemplateInfo().setEnableState(enableState).setId(id)) > 0;
    }

    @Override
    public List<ViewTemplateInfo> getList() {
        return baseMapper.selectList(new LambdaQueryWrapper<ViewTemplateInfo>().eq(ViewTemplateInfo::getEnableState,1));
    }

    @Override
    public boolean copyTemplateInfo(CopyViewTemplateVo copyViewTemplateVo) {
        ViewTemplateInfo viewTemplateInfo = copyViewTemplateVo.getViewTemplateInfo();
        viewTemplateInfo.setGroupId(copyViewTemplateVo.getGroupId());
        Map<String, String> reqMap = new HashMap<>();
        reqMap.put("sourceId",viewTemplateInfo.getId());
        reqMap.put("sourceName",viewTemplateInfo.getViewTempName()+"-"+viewTemplateInfo.getId()+".json");
        viewTemplateInfo.setId(null);
        viewTemplateInfo.setViewTempName(copyViewTemplateVo.getViewTempName());
        if (baseMapper.insert(viewTemplateInfo) > 0){
            ViewTemplateInfo templateInfo = baseMapper.selectOne(new LambdaQueryWrapper<ViewTemplateInfo>().eq(ViewTemplateInfo::getViewTempName, copyViewTemplateVo.getViewTempName()));
            reqMap.put("targetId",templateInfo.getId());
            reqMap.put("targetName",copyViewTemplateVo.getViewTempName()+templateInfo.getId()+".json");
        }
        return Boolean.parseBoolean(new Gson().fromJson(suitcaseClient.copyObject(reqMap),Result.class).getData().toString());
    }

    @Override
    public ViewTemplateInfo getViewTempInfoById(String id) {
        List<ViewTemplateInfo> list = baseMapper.selectList(new LambdaQueryWrapper<ViewTemplateInfo>()
                .eq(ViewTemplateInfo::getId, id));
        return list.isEmpty() ? new ViewTemplateInfo() : list.get(0);
    }

    private List<Map<String, String>> getGroupName(String temptype){
        Gson gson = new Gson();
        List<Map<String, String>> res = new ArrayList<>();
        List<Map<String, Object>> list = gson.fromJson(gson.toJson(gson.fromJson(suitcaseClient.getGroupNameList(temptype), Result.class).getData()), List.class);
        list.forEach(item->{
            Map<String, String> map= new HashMap<>();
            map.put(String.valueOf(item.get("id")),String.valueOf(item.get("groupName")));
            res.add(map);
        });
        return res;
    }
}
